import{_ as h,o,c as n,a as g,b as a,w as i,u as e,R as c,T as y,d as t,F as b,r as w,e as I,f as _,U as k,t as u,C as v,g as N,P as D,p as S,h as V}from"./index.f8b0a421.js";import{g as m,T as x,a as C}from"./TagsEditor.6b6e6325.js";import"./hacks.b3340c71.js";const F="/frontend/assets/data-cuate.84693c76.svg";const d=l=>(S("data-v-8399aff4"),l=l(),V(),l),L={class:"table-container-first-screen"},T=d(()=>t("h2",{class:"table-title"},"Your datasets",-1)),B={key:0,class:"datasets-table"},E=d(()=>t("thead",null,[t("tr",null,[t("th",null,"Name"),t("th",null,"Languages"),t("th",null,"Categories"),t("th",null,"Filter steps"),t("th",null,"Actions")])],-1)),R={class:"tags"},P={class:"filter-steps"},U={key:1,class:"illustration-container"},j=["src"],A=d(()=>t("p",null,"No datasets yet. Click on the import data button on the right to get started.",-1)),Y=d(()=>t("p",{class:"credits"},[_("Image by "),t("a",{href:"https://www.freepik.com/free-vector/no-data-concept-illustration_8961448.htm",target:"_blank"},"storyset on Freepik"),_(".")],-1)),q={__name:"ListDatasetsView",setup(l){function f(r){const p=new Intl.DisplayNames([],{type:"language"});return((r==null?void 0:r.columns)||[]).map(({lang:s})=>{try{return p.of(s.replace("_","-"))}catch{return s}})}return(r,p)=>(o(),n("div",L,[(o(),g(y,{to:".navbar"},[a(e(c),{class:"import-data-button",to:{name:"add-dataset-defaults"}},{default:i(()=>[_(" Import dataset "),a(e(k),{class:"import-data-icon"})]),_:1})])),T,e(m)().length>0?(o(),n("table",B,[E,t("tbody",null,[(o(!0),n(b,null,w(e(m)(),s=>(o(),n("tr",{key:s.id},[t("td",null,u(s.name),1),t("td",null,u(f(s).join("\u2013")),1),t("td",R,[a(x,{dataset:s},null,8,["dataset"])]),t("td",P,u(e(C)(s).steps.value.length),1),t("td",null,[a(e(c),{class:"icon-button",title:"Show filter yaml",to:{name:"edit-filters-yaml",params:{datasetName:s.name,format:"configuration-for-opusfilter.yaml"}}},{default:i(()=>[a(e(v))]),_:2},1032,["to"]),a(e(c),{class:"icon-button",title:"Edit filters",to:{name:"edit-filters",params:{datasetName:s.name}}},{default:i(()=>[a(e(N))]),_:2},1032,["to"]),a(e(c),{class:"icon-button",title:"Show dataset statistics",to:{}},{default:i(()=>[a(e(D))]),_:1})])]))),128))])])):(o(),n("div",U,[t("img",{src:e(F)},null,8,j),A,Y])),a(e(I))]))}},K=h(q,[["__scopeId","data-v-8399aff4"]]);export{K as default};
