.import-data-button[data-v-8399aff4]{display:flex;align-items:center;border:none;border-radius:3px;height:40px;padding:0 30px;background-color:#17223d;color:#e4960e;font-size:18px;text-decoration:none;line-height:40px}.import-data-icon[data-v-8399aff4]{margin-left:5px}.datasets-table[data-v-8399aff4]{border:2px solid #1c3948;border-collapse:collapse;width:100%}.datasets-table thead[data-v-8399aff4]{background-color:#17223d;color:#e4960e;font-size:20px;text-align:left}.datasets-table tbody[data-v-8399aff4]{color:#17223d}.datasets-table thead th[data-v-8399aff4]{padding:20px 0 10px 10px;min-width:150px}.datasets-table tbody tr[data-v-8399aff4]{border-bottom:1px solid #1c3948;border-right:2px solid #1c3948}.datasets-table tbody td[data-v-8399aff4]{border-right:2px solid #1c3948}.datasets-table tbody tr td[data-v-8399aff4]{padding:10px}.category-tags[data-v-8399aff4]{display:flex}.tags[data-v-8399aff4]{width:45%}.filter-steps[data-v-8399aff4]{text-align:right}.illustration-container[data-v-8399aff4]{text-align:center;font-size:1.2em;line-height:2}.illustration-container img[data-v-8399aff4]{max-width:600px;width:calc(100% - 4em);margin:2em}.illustration-container .credits[data-v-8399aff4]{font-size:.8em}
