.popup[data-v-fdef0114]{position:fixed;top:0;left:0;background-color:var(--background-color);border:1px solid var(--border-color);min-width:200px;box-shadow:2px 2px 8px #00000080}.popup>*[data-v-fdef0114]:not(:last-child){border-bottom:1px solid var(--border-color)}footer[data-v-fdef0114]{text-align:right}footer>*[data-v-fdef0114]:not(:last-child){margin-right:.5em}.category-list[data-v-fdef0114]{list-style:none;margin:0;padding:0}header[data-v-fdef0114],footer[data-v-fdef0114],.category-list label[data-v-fdef0114]{padding:.5em}.tags-container[data-v-d8e2ba32]{display:inline-flex;justify-content:space-between;align-items:center}.tag[data-v-d8e2ba32]{display:flex;align-items:center;width:fit-content;padding:3px 20px 3px 10px;border-radius:15px;margin-right:5px;font-size:14px;background-color:#ddd;font-family:Fira Code,monospace}.green-tag[data-v-d8e2ba32]{background-color:#afffca}.red-tag[data-v-d8e2ba32]{background-color:#ffafaf}.blue-tag[data-v-d8e2ba32]{background-color:#afcfff}
