.downloads-popup summary[data-v-f0874565]{display:flex;align-items:center;border:none;border-radius:3px;height:40px;padding:0 30px;background-color:#17223d;color:#e4960e;font-size:18px;text-decoration:none;line-height:40px}.downloads-popup[data-v-f0874565]{position:relative}.downloads-popup ul[data-v-f0874565]{position:absolute;right:0;background:white;border:1px solid #ccc;border-radius:2px;padding:10px;z-index:10;box-shadow:#00000026 0 3px 6px;overflow:hidden;overflow-y:auto;max-height:calc(100vh - 100px)}.downloads-popup li[data-v-f0874565]{list-style:none;display:flex;width:240px;height:40px;align-items:center}.downloads-popup .corpus-name[data-v-f0874565]{flex:1 1 auto;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.downloads-popup .download-state[data-v-f0874565]{margin-left:auto;padding:0 4px;opacity:.5}.downloads-popup button[data-v-f0874565]{appearance:none;background:none;border:none;margin:0 0 0 2px;cursor:pointer;opacity:.5}.downloads-popup button[data-v-f0874565]:hover,.downloads-popup button[data-v-f0874565]:active{opacity:1}.pending[data-v-f0874565]{background:#FFFFBB}.downloading[data-v-f0874565]{background:#BBFFBB}.downloaded[data-v-f0874565]{background:#BBBBFF}.loading-spinner[data-v-f5e717bd]{animation:rotate-f5e717bd 1.5s linear infinite}@keyframes rotate-f5e717bd{to{transform:rotate(360deg)}}.datasets-catalogue-title[data-v-f5e717bd]{font-size:20px;color:#182231;text-transform:uppercase;display:flex;align-items:baseline}.datasets-catalogue-title small[data-v-f5e717bd]{display:inline-block;border-left:1px solid currentColor;margin-left:10px;padding-left:10px}.datasets-catalogue-title span[data-v-f5e717bd]{font-size:16px;font-weight:lighter}.search-inputs[data-v-f5e717bd]{display:flex;margin:10px 0 20px}.search-inputs>*[data-v-f5e717bd]:not(:first-child){margin-left:3px}.search-button[data-v-f5e717bd]{color:#182231;border:none;border-radius:2px;display:inline-block;line-height:28px;height:28px;padding:0 8px;margin:0 2px;cursor:pointer;background-color:#dbe5e6}.search-button.checked[data-v-f5e717bd]{background-color:#e4960e}.search-button input[data-v-f5e717bd]{display:none}.search-inputs input[data-v-f5e717bd]{height:28px;border-radius:3px}.search-inputs input[data-v-f5e717bd]::placeholder{padding-left:5px}.search-inputs .v-select[data-v-f5e717bd]{display:inline-block;width:180px;height:28px;border-radius:3px}.search-inputs .sort-order[data-v-f5e717bd]{margin-left:auto}a.search-button[data-v-f5e717bd]{text-decoration:none}.segmented-control[data-v-f5e717bd]{display:inline-flex;margin:0 2px}.segmented-control .search-button[data-v-f5e717bd]{flex:1 1 auto;margin:0;border-radius:0}.segmented-control .search-button[data-v-f5e717bd]:first-child{border-radius:2px 0 0 2px}.segmented-control .search-button[data-v-f5e717bd]:last-child{border-radius:0 2px 2px 0}.dataset-list[data-v-f5e717bd]{display:grid;grid-template-columns:repeat(auto-fill,minmax(400px,1fr));row-gap:20px;column-gap:20px}.dataset[data-v-f5e717bd]{display:flex;flex-direction:column;justify-content:space-between;background-color:#dbe5e6;border-radius:5px;padding:20px;height:120px}.dataset-name[data-v-f5e717bd]{display:flex;justify-content:space-between}.dataset-title[data-v-f5e717bd]{font-size:26px}.metadata-dataset dt[data-v-f5e717bd]{display:none}.metadata-dataset dd[data-v-f5e717bd]{display:inline;margin-right:20px}.download-dataset-button[data-v-f5e717bd]{display:flex;align-items:center;align-self:flex-start;padding:2px 8px;border:none;border-radius:2px;background-color:#dfbd79;cursor:pointer}.download-dataset-button[data-v-f5e717bd]:disabled{cursor:default}.download-icon[data-v-f5e717bd]{margin-left:5px}
