input[type=number]{width:5em}input[type=checkbox]{width:1em}.property-list{overflow:hidden}.property-list>summary{align-items:center}.property-list>*>*:first-child{margin-left:0}.property-list>summary>button{flex:0}details.property-list>summary:before{content:" ";display:inline-block;border-top:5px solid transparent;border-bottom:5px solid transparent;border-left:5px solid currentColor;vertical-align:middle;margin-right:.7rem;transform:translateY(-2px);transition:transform .2s ease-out}details.property-list[open]>summary:before{transform:rotate(90deg) translate(-3px)}.property-list>*{padding:.4em 1em;display:flex;flex-wrap:wrap;justify-content:space-between}.property-list>summary>*,.property-list>header>*,.property-list>footer>*{flex:1;flex-wrap:nowrap}.property-list>*>*{flex:0;align-self:center;margin-left:.5em}.property-list>*>fieldset{flex:1 0 100%}.property-list>*>.property-list-description{flex:1 0 100%}.property-list>*>input[type=checkbox]{flex-basis:1em}:root{--background-color: #fff;--border-color: #17223d}*{margin:0;padding:0;box-sizing:border-box}body{margin:20px 60px;font-family:Lato,sans-serif}.navbar{display:flex;justify-content:space-between;margin-bottom:30px;align-items:center}.footer{display:flex;width:75%;margin:60px auto;font-family:Lato,sans-serif}.eu-flag,.horizon-flag{width:50px;height:auto;margin:0 10px 0 0;flex:0;align-self:flex-start}.thanking{display:flex}.version{white-space:nowrap}.icon-button{appearance:none;border:none;background:inherit;cursor:pointer;color:inherit}.icon-button:visited{color:inherit}.icon-button:disabled{cursor:inherit;opacity:.5}.modal-wrapper[data-v-1e8f5c6b]{position:fixed;top:0;left:0;width:100vw;height:100vh}.modal[data-v-1e8f5c6b]{position:absolute;z-index:1;margin:auto;left:50%;top:50%;transform:translate(-50%,-50%);background-color:#fff;padding:4em;border:1px solid #ccc;border-radius:1em;box-shadow:0 0 1em #0003}.modal-content[data-v-1e8f5c6b]{max-height:calc(100vh - 12em);overflow-y:auto}.overlay[data-v-1e8f5c6b]{z-index:1;position:absolute;width:100vw;height:100vh;background-color:#0005}.close-button[data-v-1e8f5c6b]{position:absolute;top:1em;right:1em}
