import{B as o,o as r,c as n,d as c,t as m,u as p}from"./index.f8b0a421.js";import{f as d}from"./hacks.b3340c71.js";const f={__name:"EditFiltersYamlView",setup(i){const e=o(),t=d(async a=>{if(e.params.datasetName&&e.params.format)return await(await a(`/api/datasets/${encodeURIComponent(e.params.datasetName)}/${encodeURIComponent(e.params.format)}`)).text()});return(a,s)=>(r(),n("pre",null,[c("code",null,m(p(t)),1)]))}};export{f as default};
